package org.vraptor.config;

import java.io.File;

import org.apache.log4j.Logger;
import org.vraptor.webapp.WebApplication;

public class BasePackageConfig implements ConfigItem {

	private static final Logger LOG = Logger.getLogger(BasePackageConfig.class);

	private String packageName;

	public BasePackageConfig(String pkg) {
		this.packageName = pkg;
	}

	public void register(WebApplication application) {
		LOG.warn("Deprecated: <package> is deprecated...  VRaptor will "
				+ "automatically look for @Component annotated classes if no components are defined in vraptor.xml");
		
		String lookUp = "WEB-INF/classes/" + packageName.replace('.', '/');
		LOG.debug("Looking for " + lookUp);
		String path = application.getApplicationContext().getRealPath(lookUp);
		File dir = new File(path);
		if (dir.exists() && dir.isDirectory()) {
			LOG.debug("Found " + path + " --> " + dir.getAbsolutePath());
			BasePackageComponentLoader loader = new BasePackageComponentLoader(application.getComponentManager());
			loader.loadLogics(dir, this.packageName);
		} else {
			LOG.error("Path " + path + " not found/is not a directory");
		}
	}

	public boolean isComponent() {
		return true;
	}

	public boolean isManager() {
		return false;
	}

	public String getPackage() {
		return packageName;
	}

}
