package org.vraptor.config;

import org.vraptor.component.ComponentInstantiationException;
import org.vraptor.reflection.ReflectionUtil;
import org.vraptor.url.URLManager;
import org.vraptor.webapp.WebApplication;

/**
 * Configures the url manager.
 * 
 * @author Guilherme Silveira
 * @since 2.3
 */
public class UrlManagerConfig implements ConfigItem {

	private String typeName;

	public <T> UrlManagerConfig(String typeName) {
		if (typeName == null) {
			throw new RuntimeException("Invalid url manager trying to be configured.");
		}
		this.typeName = typeName.trim();
	}

	public String getTypeName() {
		return typeName;
	}

	/**
	 * Registers itself
	 * 
	 * @see org.vraptor.config.ConfigItem#register(org.vraptor.webapp.DefaultWebApplication)
	 */
	public void register(WebApplication application) throws ConfigException {
		try {
			Object instance = ReflectionUtil.instantiate(typeName);
			if (!(instance instanceof URLManager)) {
				throw new ConfigException("Invalid url manager. It should implement the URLManager interface.");
			}
			application.setUrlManager((URLManager) instance);
		} catch (ComponentInstantiationException e) {
			throw new ConfigException(e);
		}
	}

	/**
	 * 
	 * @see org.vraptor.config.ConfigItem#isComponent()
	 */
	public boolean isComponent() {
		return false;
	}

	/**
	 * 
	 * @see org.vraptor.config.ConfigItem#isManager()
	 */
	public boolean isManager() {
		return true;
	}

}
