package org.vraptor.core;

import org.vraptor.LogicException;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.interceptor.InterceptorInstantiationException;
import org.vraptor.introspector.Introspector;
import org.vraptor.url.InvalidURLException;
import org.vraptor.url.RequestInfo;
import org.vraptor.view.ViewException;
import org.vraptor.webapp.WebApplication;

/**
 * Main vraptor2 controller.
 * 
 * @author Guilherme Silveira
 */
public class DefaultController implements Controller {

	private final WebApplication application;

	/**
	 * @param application
	 */
	public DefaultController(WebApplication application) {
		this.application = application;
	}

	/* (non-Javadoc)
	 * @see org.vraptor.core.Controller#getWebApplication()
	 */
	public WebApplication getWebApplication() {
		return application;
	}

	/* (non-Javadoc)
	 * @see org.vraptor.core.Controller#execute(org.vraptor.core.WebRequest)
	 */
	public String execute(WebRequest request) throws InvalidURLException, ComponentNotFoundException,
			LogicNotFoundException, ViewException, InterceptorInstantiationException, LogicException {
		RequestInfo requestInfo = getWebApplication().getURLManager().extractInfoFromURL(request.getRequest());
		return new VRaptorExecution(requestInfo, this, request).execute();
	}

	/* (non-Javadoc)
	 * @see org.vraptor.core.Controller#getIntrospector()
	 */
	public Introspector getIntrospector() {
		return application.getIntrospector();
	}

}
