package org.vraptor.core;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.ComponentType;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.http.VRaptorServletRequest;
import org.vraptor.interceptor.ComponentLookupInterceptor;
import org.vraptor.interceptor.ExecuteLogicInterceptor;
import org.vraptor.interceptor.InjectionInterceptor;
import org.vraptor.interceptor.InterceptorDealer;
import org.vraptor.interceptor.InterceptorInstantiationException;
import org.vraptor.interceptor.InterceptorType;
import org.vraptor.interceptor.OutjectionInterceptor;
import org.vraptor.interceptor.RegisterAttributesInteceptor;
import org.vraptor.interceptor.SettingAndValidationInterceptor;
import org.vraptor.interceptor.ViewInterceptor;
import org.vraptor.scope.DefaultLogicRequest;
import org.vraptor.url.RequestInfo;
import org.vraptor.view.ViewException;

/**
 * VRaptor's engine.
 *
 * @author Guilherme Silveira
 * @author Paulo Silveira
 */
@SuppressWarnings("unchecked")
public class VRaptorExecution {

	private static final Logger LOG = Logger.getLogger(VRaptorExecution.class);

	private final Controller controller;

	private final RequestInfo requestInfo;

	private final WebRequest webRequest;

	private final static Class<Interceptor>[] AFTER = new Class[] { ComponentLookupInterceptor.class,
			InjectionInterceptor.class, SettingAndValidationInterceptor.class, ExecuteLogicInterceptor.class,
			OutjectionInterceptor.class, ViewInterceptor.class };

	public VRaptorExecution(RequestInfo requestInfo, Controller controller, WebRequest webRequest) {
		this.webRequest = webRequest;
		this.controller = controller;
		this.requestInfo = requestInfo;
	}

	public String execute() throws ComponentNotFoundException, LogicNotFoundException, ViewException,
			InterceptorInstantiationException, LogicException {

		if (LOG.isDebugEnabled()) {
			LOG.debug("Calling execute on " + requestInfo.toString());
		}

		ComponentType componentType = this.controller.getWebApplication().getComponentManager().getComponent(
				requestInfo);
		DefaultLogicDefinition definition = new DefaultLogicDefinition(null, componentType, componentType.getLogic(requestInfo.getLogicName()));

		VRaptorServletRequest vraptorRequest = new VRaptorServletRequest(webRequest.getRequest(), controller
				.getWebApplication().getFactoryManager(), controller.getWebApplication().getIntrospector());

		WebRequest wrappedWebRequest = new WebRequest(vraptorRequest, webRequest.getResponse(), webRequest.getContext());
		DefaultLogicRequest logicRequest = new DefaultLogicRequest(requestInfo, wrappedWebRequest, definition);

		// TODO: are these necessary? at least externalize
		// FIXME: vraptorRequest and logcRequest have a bidirection dependency
		vraptorRequest.setCurrentRequest(logicRequest);
		vraptorRequest.setAttribute("context", logicRequest);
		vraptorRequest.setAttribute("application", controller.getWebApplication().getApplicationContext());

		List<Interceptor> before = new ArrayList<Interceptor>();
		before.add(new RegisterAttributesInteceptor());

		List<InterceptorType> fullInterceptors = new ArrayList<InterceptorType>(componentType.getInterceptors());
		for (Class<? extends Interceptor> type : AFTER) {
			fullInterceptors.add(InterceptorType.getType(type));
		}
		InterceptorDealer dealer = new InterceptorDealer(logicRequest, controller.getIntrospector(), before,
				fullInterceptors, new ArrayList<Interceptor>());
		InterceptorsLogicFlow logicFlow = new InterceptorsLogicFlow(logicRequest, dealer);

		logicFlow.execute();

		return logicFlow.getLogicRequest().getResult();
	}

}
